#!/usr/bin/env python
# -*- coding: utf-8 -*-

from os import system

import pygtk
pygtk.require("2.0")
import gtk
from cgaVentanaCambioContrasena import cambio_contrasena
from cgapwdmanager import pwdManager
from cgaconfig import *
import cgaManejadorVentana


class App_controller(gtk.Window):

    def __init__(self):
        """
            Asignamos a los manejadores de los botones de las ventanas login y
            filtro sus funciones correspondientes.
        """
        
        self.mv=cgaManejadorVentana.Controlador_Ventanas()
        self.pm= pwdManager()
        
        self.mv.vl_asignar_enter_entry_pw(self.autorizar)
        self.mv.vl_asignar_manejador_btn_auto(self.autorizar)
        self.mv.vl_asignar_manejador_btn_salir(self.salir)
        self.mv.vf_asignar_manejador_btn_aplicar(self.aplicar)
        self.mv.vf_asignar_manejador_btn_cancelar(self.confirmar_salir)    
        self.mv.vf_asignar_manejador_btn_aceptar(self.aceptar)
        self.mv.vf_asignar_manejador_btn_change_pwd(self.change_pwd)
        self.mv.vf_asignar_manejador_btn_enable(self.enable_disable_privoxy)
        
        # Comprobamos si el filtro esta activado o desactivado
        with open(PRIVOXY_CGACONFIG, "r") as f:
            for linea in f.readlines():
                linea=linea.strip().split("=")
                if linea[0] == "PRIVOXY_RUN":
                    self.privoxy_enabled = linea[1] == "yes"

        #Usamos set_use_markup(True) para poder darle formato al texto
        
        self.mv.vl.lbl_info.set_use_markup(True)
        
        #definimos en el formato el color verde para cuando esta activado 
        #y el color rojo para cuando esta desactivado 
        formato = "<span weight='bold' foreground='%s'>%s</span>"\
        %[("red", "Desactivado"),("#008532", "Activado")][self.privoxy_enabled]
        
        #Mostrar el estado del filtro en la ventana filtro 
        self.mv.vf.e_estado.set_label("Estado del filtro: %s" % formato)
        #Mostrar el estado del filtro en la ventana login
        self.mv.vl.lbl_info.set_label("Estado del filtro: %s" % formato)
        
    
    def set_autorizado(self, autorizado):
        #Una vez autorizado mostrar el valor de la lista blanca y
        #la lista negra
        self.autorizado=autorizado
        self.update_ui()
         

    def update_ui(self):
        #Muestra los botones que se usan para administrar el filtro y 
        #las listas 
        l_widgets=[ self.mv.vf.b_aceptar, self.mv.vf.b_aplicar,\
                    self.mv.vf.b_change_pwd, self.mv.vf.t_edicion_negro,\
                    self.mv.vf.t_edicion_blanco, self.mv.vf.b_enabled ]

        for w in l_widgets:
            w.set_sensitive(self.autorizado)
        
        #Muestra los valores de la lista blanca y lista negra
        if self.autorizado:
            self.load_listas()
            self.mv.vf.contenido_negro.\
            place_cursor(self.mv.vf.contenido_negro.get_start_iter())
            self.mv.vf.contenido_negro.connect("changed", self.modificar,\
                                               "negra")
            self.mv.vf.contenido_blanco.\
            place_cursor(self.mv.vf.contenido_blanco.get_start_iter())
            self.mv.vf.contenido_blanco.connect("changed", self.modificar,\
                                                "blanca")

        self.mv.vf.b_auth.set_sensitive(not self.autorizado)

        self.update_enable_disable_button()

    def update_enable_disable_button(self):
        # Función del boton activar y desactivar el filtro
        if self.privoxy_enabled:
            self.mv.vf.b_enabled.set_label("Desactivar filtrado")
          
        else:
            self.mv.vf.b_enabled.set_label("Activar filtrado")
        
        formato = "<span weight='bold' foreground='%s'>%s</span>"\
        %[("red", "Desactivado"),("#008532", "Activado")][self.privoxy_enabled]
        
        self.mv.vf.e_estado.set_label("Estado del filtro: %s" % formato)

    def enable_disable_privoxy(self, boton):
        #Desactivar el filtro
        self.privoxy_enabled = not self.privoxy_enabled
        self.update_enable_disable_button()
        self.update_privoxy_config_file()

    def update_privoxy_config_file(self):
        #Modificar el filtro
        estado = ["no", "yes"][self.privoxy_enabled]
        with open(PRIVOXY_CGACONFIG, "w") as f:
            f.write("PRIVOXY_RUN=%s" % estado)
        self.restart_service()

    def load_listas(self):
        #Cargar las listas del filtro
        try:
            blanco=open(WHITELIST, "r").read()
        except IOError:
            blanco=""
        try:
            negro=open(BLACKLIST, "r").read()
        except IOError:
            negro=""
    
        self.mv.vf.contenido_blanco.set_text(blanco)
        self.mv.vf.contenido_negro.set_text(negro)

    def modificar(self, w, cual):
        self.mv.vf.modificado[cual]=True

    def save_file(self):
        """ 
            Guarda la plantilla con las listas configuradas por el usuario 
        """
        
        datos = [(self.mv.vf.contenido_negro,  PLANTILLA_BLACK, CONF_BLACK,\
                  BLACKLIST ),
                 (self.mv.vf.contenido_blanco, PLANTILLA_WHITE, CONF_WHITE,\
                  WHITELIST ) ]

        for (w, plantilla, conf, cache)  in datos:
            cont=w.get_text(w.get_start_iter(),w.get_end_iter())
            header=open(plantilla, "r").read()
            with open(conf, "w") as f:
                f.write(header)
                f.write("\n")
                f.write(cont)
            with open(cache, "w") as f:
                f.write(cont)

        self.mv.vf.modificado["blanca"]=self.mv.vf.modificado["negra"]=False

    def aplicar(self, param=None):
        #Guarda los valores de las listas
        if any(self.mv.vf.modificado.values()):
            self.save_file()
            self.restart_service()

    def aceptar(self, _):
        #Guarda los valores de las listas y cierra el programa
        self.aplicar()
        gtk.main_quit()

    def confirmar_salir(self, *args):
        #Cierra sin guardar los cambios en las listas
        if any(self.mv.vf.modificado.values()):
            self.pregunta_salir()
        else:
            gtk.main_quit()

    def pregunta_salir(self):
        #Mensaje de advertencia cuando intentas salir sin guardar los cambios
        mensaje="Hay cambios sin guardar, ¿Seguro que desea salir?"
        m=gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING,\
                            gtk.BUTTONS_YES_NO, mensaje)
        res=m.run()
        m.hide()
        if res == gtk.RESPONSE_YES:
            gtk.main_quit()
        
    def restart_service(self):
        #Reiniciar el servicio para que se apliquen las modificaciones 
        #de las listas
        print "Reiniciando servicio"
        comando="/etc/init.d/privoxy-cgaconfig restart"
        ret=system(comando)
        if ret==0:
            print "Éxito reiniciando privoxy"
        else:
            print "No se pudo reiniciar privoxy"
            
   
    def salir (self,widget=None):
        """
            Procedimento para salir del programa
        """
        gtk.main_quit()
        
    def autorizar(self,widget=None):
        """
            Comprobar si el usuario es el autorizado
        """
        #el valor será true si es el correcto
        
        valor = self.pm.check(self.mv.vl.entry_pw.get_text())
        
        #Si es el autorizado carga las listasd, cierra la ventana login y 
        #muestra la ventana filtro 
        
        if valor:
            self.autorizado=True
            self.update_ui()
            self.mv.vf.show_all()
            self.mv.vl.frm_main.destroy()
        
        #Si no es el usuario autorizado muestra un mensaje de error y vuelve 
        #a solicitar el usuario
            
        else: 
            self.mv.vl.entry_pw.set_text("")
            m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,\
                                  gtk.BUTTONS_OK, "Contraseña erronea")
            res=m.run()
            m.hide()

    def change_pwd(self, widget):
        """
            Cambio la contraseña del usuario autorizado
        """
        p=cambio_contrasena(parent=self)
        res=p.run()
        if res== gtk.RESPONSE_OK:
            if p.iguales():
                self.pm.update(p.get_pass())
                self.pm.save()
                
                #Muestra mensaje de información si se ha cambiado 
                #correctamente
                 
                m=gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO,\
                                    gtk.BUTTONS_OK, "Contraseña cambiada"+
                                    " correctamente.")
                res=m.run()
                m.hide()
            else:
                
                #Muestra mensaje de error si no se ha cambiado 
                #correctamente
                
                m=gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,\
                                    gtk.BUTTONS_OK, "Las constraseñas no"+
                                    " coinciden.")
                res=m.run()
                m.hide()
        p.hide()

if __name__== "__main__":
    #llamada al programa completo
    c=App_controller()
    gtk.main()
